@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,6 @@
 import logging, re
 from collections import OrderedDict
 
-from django.core.exceptions import FieldError
 from django.db import transaction
 from django.db.models import F, Max, Q
 from django.http import Http404
@@ -35,6 +34,7 @@ from rest_framework.mixins import CreateModelMixin
 from rest_framework.filters import OrderingFilter, SearchFilter
 from rest_framework.pagination import PageNumberPagination
 
+from .. import settings
 from ..compat import reverse
 from ..mixins import AccountMixin, MatrixMixin
 from ..models import (Answer, Matrix, EditableFilter,
@@ -42,11 +42,37 @@ from ..models import (Answer, Matrix, EditableFilter,
 from ..utils import (get_account_model, get_account_serializer,
     get_question_serializer)
 from .serializers import (AccountsFilterAddSerializer,
-    EditableFilterSerializer, MatrixSerializer)
+    EditableFilterSerializer, MatrixSerializer, SampleAnswerSerializer)
 
 
 LOGGER = logging.getLogger(__name__)
 
+class CompareAPIView(generics.ListAPIView):
+    """
+    Lists compared samples
+
+    **Examples**:
+
+    .. code-block:: http
+
+        GET /api/energy-utility/reporting/sustainability/matrix/compare HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+
+        {
+          "count": 4,
+          "results": []
+        }
+    """
+    serializer_class = SampleAnswerSerializer
+
+    def get_queryset(self):
+        # XXX TODO implement!!!
+        return Answer.objects.none()
+
 
 class MatrixCreateAPIView(generics.ListCreateAPIView):
     """
@@ -891,8 +917,11 @@ class AccountsFilterListAPIView(EditableFilterObjectsAPIView):
             "results": [
             {
                "slug": "supplier-1",
-               "full_name": "Supplier 1",
-               "email": "supplier-1@localhost.localdomain"
+               "title": "Supplier 1",
+               "extra": null,
+               "predicates": [],
+               "likely_metric": [],
+               "results": []
            }]
         }
     """
@@ -913,6 +942,7 @@ class AccountsFilterListAPIView(EditableFilterObjectsAPIView):
         .. code-block:: json
 
             {
+              "title": "Supplier 1"
             }
 
         responds
@@ -921,8 +951,11 @@ class AccountsFilterListAPIView(EditableFilterObjectsAPIView):
 
             {
                "slug": "supplier-1",
-               "printable_name": "Supplier 1",
-               "extra": null
+               "title": "Supplier 1",
+               "extra": null,
+               "predicates": [],
+               "likely_metric": [],
+               "results": []
             }
         """
         #pylint:disable=useless-super-delegation
