@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,6 +26,7 @@
 import decimal, json, logging
 from collections import OrderedDict
 
+from django.contrib.auth import get_user_model
 from django.db import transaction
 from django.db.models import Max
 from django.db.utils import DataError
@@ -41,7 +42,8 @@ from ..mixins import AccountMixin, SampleMixin
 from ..models import Answer, Choice, Sample, Unit, UnitEquivalences
 from .serializers import (AnswerSerializer, NoModelSerializer,
     SampleAnswerSerializer, SampleCreateSerializer, SampleSerializer)
-from ..utils import datetime_or_now, get_question_model, is_sqlite3
+from ..utils import (datetime_or_now, get_question_model, get_user_serializer,
+    is_sqlite3)
 
 
 LOGGER = logging.getLogger(__name__)
@@ -360,7 +362,7 @@ WITH answers AS (
       survey_answer.collected_by_id AS collected_by_id,
       survey_answer.sample_id AS sample_id,
       COALESCE(survey_choice.text,
-        survey_answer.measured%(convert_to_text)s) AS measured_text
+        survey_answer.measured%(convert_to_text)s) AS _measured_text
     FROM survey_answer
     INNER JOIN survey_question
       ON survey_answer.question_id = survey_question.id
@@ -423,7 +425,7 @@ SELECT
     answers.sample_id AS sample_id,
     questions.rank AS _rank,
     questions.required AS required,
-    answers.measured_text AS measured_text
+    answers._measured_text AS _measured_text
 FROM questions
 LEFT OUTER JOIN answers
   ON questions.id = answers.question_id""" % {
@@ -446,7 +448,7 @@ WITH answers AS (
       survey_answer.collected_by_id AS collected_by_id,
       survey_answer.sample_id AS sample_id,
       COALESCE(survey_choice.text,
-        survey_answer.measured%(convert_to_text)s) AS measured_text
+        survey_answer.measured%(convert_to_text)s) AS _measured_text
     FROM survey_answer
     LEFT OUTER JOIN survey_choice
       ON survey_choice.id = survey_answer.measured
@@ -476,7 +478,7 @@ SELECT
     answers.sample_id AS sample_id,
     campaign_questions.rank AS _rank,
     campaign_questions.required AS required,
-    answers.measured_text AS measured_text
+    answers._measured_text AS _measured_text
 FROM campaign_questions
 LEFT OUTER JOIN answers
   ON campaign_questions.id = answers.question_id""" % {
@@ -850,7 +852,7 @@ class SampleCandidatesMixin(SampleMixin):
             content_answers.collected_by_id AS collected_by_id,
             content_answers.sample_id AS sample_id,
             COALESCE(survey_choice.text,
-              content_answers.measured%(convert_to_text)s) AS measured_text
+              content_answers.measured%(convert_to_text)s) AS _measured_text
           FROM content_answers INNER JOIN latest_answers
             ON (content_answers.content_id = latest_answers.content_id
             AND content_answers.unit_id = latest_answers.unit_id
@@ -871,7 +873,7 @@ class SampleCandidatesMixin(SampleMixin):
           candidate_answers.sample_id AS sample_id,
           survey_enumeratedquestions.rank AS _rank,
           survey_enumeratedquestions.required AS required,
-          candidate_answers.measured_text AS measured_text
+          candidate_answers._measured_text AS _measured_text
         FROM survey_question
         INNER JOIN survey_enumeratedquestions
           ON survey_enumeratedquestions.question_id = survey_question.id
@@ -1419,6 +1421,38 @@ class SampleResetAPIView(SampleMixin, generics.CreateAPIView):
             headers=headers)
 
 
+class SampleResetIndexAPIView(SampleResetAPIView):
+    """
+    Clears all answers
+
+    The ``sample`` must belong to ``organization``.
+
+    ``path`` can be used to filter the tree of questions by a prefix.
+
+    **Tags**: assessments
+
+    **Examples**
+
+    .. code-block:: http
+
+        POST /api/supplier-1/sample/46f66f70f5ad41b29c4df08f683a9a7a/reset\
+ HTTP/1.1
+
+    .. code-block:: json
+
+        {}
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "slug": "46f66f70f5ad41b29c4df08f683a9a7a",
+            "created_at": "2018-01-24T17:03:34.926193Z",
+            "campaign": "sustainability"
+        }
+    """
+
 class SampleRecentCreateAPIView(AccountMixin, generics.ListCreateAPIView):
     """
     Lists updatable samples
@@ -1495,3 +1529,83 @@ class SampleRecentCreateAPIView(AccountMixin, generics.ListCreateAPIView):
 
     def perform_create(self, serializer):
         serializer.save(account=self.account)
+
+
+class SampleRespondentsAPIView(SampleMixin, generics.ListAPIView):
+    """
+    Lists unique respondents
+
+    The list returned contains the information about the users who answered
+    at least one question.
+
+    The {sample} must belong to {organization}.
+
+    {path} can be used to filter the tree of questions by a prefix.
+
+    **Tags**: assessments
+
+    **Examples**
+
+    .. code-block:: http
+
+         GET /api/supplier-1/sample/46f66f70f5ad41b29c4df08f683a9a7a\
+/respondents/construction HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+    {
+        "count": 1,
+        "previous": null,
+        "next": null,
+        "results": [{
+            "slug": "steve",
+            "full_name": "Steve"
+         }]
+    }
+    """
+    serializer_class = get_user_serializer()
+
+    def get_queryset(self):
+        kwargs = {}
+        if self.path:
+            kwargs = {'answer__question__path__startwith': self.path}
+        return get_user_model().objects.filter(
+            answer__sample=self.sample, **kwargs).distinct()
+
+
+class SampleRespondentsIndexAPIView(SampleRespondentsAPIView):
+    """
+    Lists unique respondents
+
+    The list returned contains the information about the users who answered
+    at least one question.
+
+    The {sample} must belong to {organization}.
+
+    {path} can be used to filter the tree of questions by a prefix.
+
+    **Tags**: assessments
+
+    **Examples**
+
+    .. code-block:: http
+
+         GET /api/supplier-1/sample/46f66f70f5ad41b29c4df08f683a9a7a\
+/respondents HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+    {
+        "count": 1,
+        "previous": null,
+        "next": null,
+        "results": [{
+            "slug": "steve",
+            "full_name": "Steve"
+         }]
+    }
+    """
