@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -21,6 +21,7 @@
 # WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+import json
 
 from django.db import transaction
 from django.template.defaultfilters import slugify
@@ -30,7 +31,6 @@ from rest_framework.generics import get_object_or_404
 
 from .. import settings
 from ..compat import gettext_lazy as _, reverse, six
-from ..helpers import extra_as_internal
 from ..models import (Answer, Campaign, Choice,
     EditableFilter, EditablePredicate, Matrix, PortfolioDoubleOptIn,
     Sample, Unit)
@@ -72,6 +72,21 @@ class EnumField(serializers.Field):
         return result
 
 
+class ExtraField(serializers.CharField):
+
+    def to_internal_value(self, data):
+        if isinstance(data, dict):
+            return json.dumps(data)
+        return super(ExtraField, self).to_internal_value(data)
+
+    def to_representation(self, value):
+        try:
+            return json.loads(value)
+        except (TypeError, ValueError):
+            pass
+        return super(ExtraField, self).to_representation(value)
+
+
 class NoModelSerializer(serializers.Serializer):
 
     def create(self, validated_data):
@@ -88,7 +103,7 @@ class AccountSerializer(serializers.ModelSerializer):
         help_text=_("Name that can be safely used for display in HTML pages"))
     picture = serializers.SerializerMethodField(read_only=True,
         help_text=_("URL location of the profile picture"))
-    extra = serializers.SerializerMethodField(read_only=True,
+    extra = ExtraField(read_only=True,
         help_text=_("Extra meta data (can be stringify JSON)"))
     rank = serializers.SerializerMethodField(read_only=True,
         help_text=_("rank in filter"))
@@ -99,8 +114,10 @@ class AccountSerializer(serializers.ModelSerializer):
 
     @staticmethod
     def get_slug(obj):
-        if hasattr(obj, 'slug'):
+        try:
             return obj.slug
+        except AttributeError:
+            pass
         return obj.username
 
     @staticmethod
@@ -115,15 +132,10 @@ class AccountSerializer(serializers.ModelSerializer):
     def get_picture(obj):
         try:
             return obj.picture
-        except:
+        except AttributeError:
             pass
         return None
 
-    @staticmethod
-    def get_extra(obj):
-        extra = extra_as_internal(obj)
-        return extra
-
     @staticmethod
     def get_rank(obj):
         try:
@@ -180,10 +192,15 @@ class UnitSerializer(serializers.ModelSerializer):
 
 class QuestionCreateSerializer(serializers.ModelSerializer):
 
-    title = serializers.CharField(allow_blank=True)
+    title = serializers.CharField(allow_blank=True,
+        help_text=_("Title of the question as displayed in user interfaces"))
+    text = serializers.CharField(required=False, allow_blank=True,
+        help_text=_("Long form description about the question"))
     default_unit = serializers.SlugRelatedField(slug_field='slug',
         queryset=Unit.objects.all(),
         help_text=_("Default unit for measured field when none is specified"))
+    extra = ExtraField(required=False,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     class Meta:
         model = get_question_model()
@@ -362,6 +379,8 @@ class EditableFilterSerializer(serializers.ModelSerializer):
     likely_metric = serializers.SerializerMethodField()
     predicates = EditablePredicateSerializer(many=True, required=False)
     results = get_account_serializer()(many=True, required=False) # XXX do we still need this field?
+    extra = ExtraField(required=False, allow_null=True, allow_blank=True,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     class Meta:
         model = EditableFilter
@@ -384,6 +403,8 @@ class EditableFilterSerializer(serializers.ModelSerializer):
                     operand=predicate['operand'],
                     field=predicate['field'],
                     selector=predicate['selector'])
+                # We have a reverse ORM relationship called 'predicates'
+                #pylint:disable=no-member
                 editable_filter.predicates.add(predicate)
         return editable_filter
 
@@ -392,8 +413,8 @@ class EditableFilterSerializer(serializers.ModelSerializer):
             instance.title = validated_data['title']
             instance.extra = validated_data['extra']
             instance.save()
-            absents = set([item['pk']
-                for item in instance.predicates.all().values('pk')])
+            absents = set(instance.predicates.all().values_list(
+                'pk', flat=True))
             for idx, predicate in enumerate(validated_data['predicates']):
                 predicate, _ = EditablePredicate.objects.get_or_create(
                     editable_filter=instance,
@@ -426,7 +447,7 @@ class MatrixSerializer(serializers.ModelSerializer):
             editable_filter_serializer = EditableFilterSerializer()
             for cohort in validated_data['cohorts']:
                 cohort = editable_filter_serializer.create(cohort)
-                matrix.predicates.add(cohort)
+                matrix.cohorts.add(cohort)
         return matrix
 
     def update(self, instance, validated_data):
@@ -437,8 +458,7 @@ class MatrixSerializer(serializers.ModelSerializer):
                     EditableFilter.objects.all(),
                     slug=validated_data['metric']['slug'])
             instance.save()
-            absents = set([item['pk']
-                for item in instance.cohorts.all().values('pk')])
+            absents = set(instance.cohorts.all().values_list('pk', flat=True))
             for cohort in validated_data['cohorts']:
                 cohort = get_object_or_404(
                     EditableFilter.objects.all(), slug=cohort['slug'])
@@ -450,15 +470,18 @@ class MatrixSerializer(serializers.ModelSerializer):
 
 class KeyValueTuple(serializers.ListField):
 
-    child = serializers.CharField() # XXX (String, Integer)
+    child = serializers.CharField(allow_null=True) # XXX (String, Integer)
     min_length = 3
     max_length = 3
 
 
 class TableSerializer(NoModelSerializer):
 
-    key = serializers.CharField(
+    slug = serializers.SlugField(
         help_text=_("Unique key in the table for the data series"))
+    printable_name = serializers.CharField(
+        required=False, read_only=True,
+        help_text=_("Name that can be safely used for display in HTML pages"))
     values = serializers.ListField(
         child=KeyValueTuple(),
         help_text="Datapoints in the serie")
@@ -468,12 +491,13 @@ class MetricsSerializer(NoModelSerializer):
 
     scale = serializers.FloatField(required=False,
         help_text=_("The scale of the number reported in the tables (ex: 1000"\
-        " when numbers are reported in thousands of dollars)"))
-    unit = serializers.CharField(required=False,
-        help_text=_("Three-letter ISO 4217 code for currency unit (ex: usd)"))
+        " when numbers are reported in thousands)"))
+    unit = serializers.SlugRelatedField(required=False, allow_null=True,
+        queryset=Unit.objects.all(), slug_field='slug',
+        help_text=_("Unit the measured field is in"))
     title = serializers.CharField(
         help_text=_("Title for the table"))
-    table = TableSerializer(many=True)
+    results = TableSerializer(many=True)
 
 
 class InviteeSerializer(NoModelSerializer):
@@ -485,7 +509,11 @@ class InviteeSerializer(NoModelSerializer):
 
     @staticmethod
     def get_printable_name(obj):
-        return str(obj.full_name)
+        try:
+            return str(obj.full_name)
+        except AttributeError:
+            pass
+        return obj.get('full_name')
 
 
 class PortfolioGrantCreateSerializer(serializers.ModelSerializer):
@@ -510,11 +538,23 @@ class PortfolioRequestCreateSerializer(serializers.ModelSerializer):
     campaign = serializers.SlugRelatedField(required=False,
         queryset=Campaign.objects.all(), slug_field='slug')
     message = serializers.CharField(required=False, allow_null=True)
+    extra = ExtraField(required=False,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     class Meta:
         model = PortfolioDoubleOptIn
-        fields = ("accounts", "campaign", "message", "ends_at",)
-        read_only_fields = ("ends_at",)
+        fields = ('accounts', 'campaign', 'message', 'ends_at', 'extra')
+        read_only_fields = ('ends_at',)
+
+
+class PortfolioOptInUpdateSerializer(serializers.ModelSerializer):
+
+    extra = ExtraField(required=False,
+        help_text=_("Extra meta data (can be stringify JSON)"))
+
+    class Meta:
+        model = PortfolioDoubleOptIn
+        fields = ('extra',)
 
 
 class PortfolioOptInSerializer(serializers.ModelSerializer):
@@ -530,12 +570,15 @@ class PortfolioOptInSerializer(serializers.ModelSerializer):
     state = EnumField(choices=PortfolioDoubleOptIn.STATES)
     api_accept = serializers.SerializerMethodField()
     api_remove = serializers.SerializerMethodField()
+    extra = ExtraField(required=False,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     class Meta:
         model = PortfolioDoubleOptIn
-        fields = ('grantee', 'account', 'campaign', 'ends_at',
+        fields = ('grantee', 'account', 'campaign', 'created_at', 'ends_at',
+            'state', 'api_accept', 'api_remove', 'extra')
+        read_only_fields = ('created_at', 'ends_at',
             'state', 'api_accept', 'api_remove')
-        read_only_fields = ('ends_at', 'state', 'api_accept', 'api_remove')
 
     def get_api_accept(self, obj):
         api_endpoint = None
@@ -574,7 +617,8 @@ class AccountsFilterAddSerializer(serializers.ModelSerializer):
 
     slug = serializers.CharField(required=False)
     full_name = serializers.CharField()
-    extra = serializers.CharField(required=False)
+    extra = ExtraField(required=False,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     class Meta:
         model = get_account_model()
