@@ -96,7 +96,7 @@ class AccountMixin(object):
 
     def get_context_data(self, **kwargs):
         context = super(AccountMixin, self).get_context_data(**kwargs)
-        context.update({'account': self.account})
+        context.update({self.account_url_kwarg: self.account})
         return context
 
     def get_reverse_kwargs(self):
@@ -193,6 +193,14 @@ class CampaignMixin(object):
                 self._db_path = self.DB_PATH_SEP + self._db_path
         return self._db_path
 
+    def get_reverse_kwargs(self):
+        """
+        List of kwargs taken from the url that needs to be passed through
+        to ``get_success_url``.
+        """
+        return super(CampaignMixin, self).get_reverse_kwargs() + [
+            self.campaign_url_kwarg]
+
 
 class CampaignQuerysetMixin(BelongsMixin):
 
@@ -337,7 +345,7 @@ class SampleMixin(AccountMixin, QuestionMixin):
                         (Q(account__portfolios__grantee=self.account) &
                          Q(account__portfolios__ends_at__gte=F('created_at'))),
                         slug=sample_slug)
-                sample = queryset.select_related('campaign').get()
+                sample = queryset.select_related('campaign').distinct().get()
             except Sample.DoesNotExist:
                 raise Http404("Cannot find Sample(slug='%s')" % sample_slug)
         else:
