@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -22,7 +22,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import hashlib, random, uuid
+import datetime, hashlib, random, uuid
 
 from django.db import models, transaction, IntegrityError
 from django.template.defaultfilters import slugify
@@ -129,11 +129,22 @@ class Unit(models.Model):
         return str(self.slug)
 
 
+class UnitEquivalencesMaganger(models.Manager):
+
+    def are_equiv(self, left, right):
+        return self.filter(
+            (models.Q(source=left) & models.Q(target=right)) |
+            (models.Q(source=right) & models.Q(target=left))
+        ).exists()
+
+
 @python_2_unicode_compatible
 class UnitEquivalences(models.Model):
     """
     Pairs of units that can be translated one to the other.
     """
+    objects = UnitEquivalencesMaganger()
+
     source = models.ForeignKey(Unit, on_delete=models.CASCADE,
         related_name='source_equivalences',
         help_text=_("Source unit in the equivalence"))
@@ -158,8 +169,12 @@ class Choice(models.Model):
         on_delete=models.PROTECT, null=True)
     rank = models.IntegerField(
         help_text=_("used to order choice when presenting a question"))
-    text = models.TextField()
-    descr = models.TextField()
+    text = models.TextField(
+        help_text=_("short name for the enumerated value"\
+        " (as used in source code)"))
+    descr = models.TextField(
+        help_text=_("long form description of the enumerated value"\
+        " (for help tooltips)"))
 
     class Meta:
         unique_together = ('unit', 'question', 'rank')
@@ -213,7 +228,8 @@ class AbstractQuestion(SlugTitleMixin, models.Model):
     path = models.CharField(max_length=1024, unique=True, db_index=True,
         help_text=_("Unique identifier that can be used in URL"))
     content = models.ForeignKey(settings.CONTENT_MODEL,
-        on_delete=models.PROTECT, related_name='question')
+        on_delete=models.PROTECT, related_name='question',
+        help_text=_("Title, description and metadata about the question"))
     ui_hint = models.CharField(
         max_length=20, choices=UI_HINTS, default=RADIO,
         help_text=_("Choose the type of answser."))
@@ -270,9 +286,6 @@ class AbstractQuestion(SlugTitleMixin, models.Model):
 @python_2_unicode_compatible
 class Question(AbstractQuestion):
 
-# XXX Before migration:
-#    pass
-# XXX After migration
     class Meta(AbstractQuestion.Meta):
         swappable = 'QUESTION_MODEL'
 
@@ -287,11 +300,13 @@ class Campaign(SlugTitleMixin, models.Model):
         help_text=_("Unique identifier that can be used in a URL"))
     created_at = models.DateTimeField(auto_now_add=True)
     title = models.CharField(max_length=150,
-        help_text=_("Enter a campaign title."))
+        help_text=_("Title of the campaign as displayed in user interfaces"))
     description = models.TextField(null=True, blank=True,
-        help_text=_("This description will be displayed to interviewees."))
+        help_text=_("Long form description displayed to users responding"\
+        " to the campaign"))
     account = models.ForeignKey(settings.BELONGS_MODEL,
-        on_delete=models.PROTECT, null=True)
+        on_delete=models.PROTECT, null=True,
+        help_text=_("Acccount that can make edits to the campaign"))
     active = models.BooleanField(default=False,
         help_text=_("Whether the campaign is available or not"))
     quizz_mode = models.BooleanField(default=False,
@@ -304,7 +319,8 @@ class Campaign(SlugTitleMixin, models.Model):
     questions = models.ManyToManyField(settings.QUESTION_MODEL,
         through='survey.EnumeratedQuestions', related_name='campaigns',
         help_text=_("Questions which are part of the campaign"))
-    extra = get_extra_field_class()(null=True)
+    extra = get_extra_field_class()(null=True, blank=True,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     def __str__(self):
         return str(self.slug)
@@ -343,46 +359,67 @@ class SampleManager(models.Manager):
         return self.create(account=get_account_model().objects.get(
                 **account_lookup_kwargs), **kwargs)
 
-    def get_latest_frozen_by_accounts(self, campaign,
-                                      before=None, excludes=None):
+    def get_latest_frozen_by_accounts(self, campaign=None,
+                                      start_at=None, ends_at=None,
+                                      tags=None, pks_only=False):
         """
-        Returns the most recent frozen assessment before an optionally specified
-        date, indexed by account.
+        Returns the most recent frozen sample in an optionally specified
+        date range, indexed by account.
 
-        All accounts in ``excludes`` are not added to the index. This is
-        typically used to filter out 'testing' accounts
+        The returned queryset can be further filtered by a campaign and
+        a set of tags.
         """
-        if excludes:
-            if isinstance(excludes, list):
-                excludes = ','.join([
-                    str(account_id) for account_id in excludes])
-            filter_out_testing = (
-                "AND survey_sample.account_id NOT IN (%s)" % str(excludes))
+        #pylint:disable=too-many-arguments
+        campaign_clause = ""
+        if campaign:
+            campaign_clause = (
+                "AND survey_sample.campaign_id = %(campaign_id)d" % {
+                    'campaign_id': campaign.pk})
+        date_range_clause = ""
+        if start_at:
+            date_range_clause = (" AND survey_sample.created_at >= '%s'" %
+                start_at.isoformat())
+        if ends_at:
+            date_range_clause += (" AND survey_sample.created_at < '%s'" %
+                ends_at.isoformat())
+        extra_clause = ""
+        if tags is not None:
+            if tags:
+                extra_clause = "".join([
+                    "AND LOWER(survey_sample.extra) LIKE '%%%s%%'" %
+                    tag.lower() for tag in tags])
+            else:
+                extra_clause = "AND survey_sample.extra IS NULL"
+        if pks_only:
+            values = 'survey_sample.id'
         else:
-            filter_out_testing = ""
-        before_clause = ("AND created_at < '%s'" % before.isoformat()
-            if before else "")
+            values = 'survey_sample.*'
+
         sql_query = """SELECT
-    survey_sample.account_id AS account_id,
-    survey_sample.id AS id,
-    survey_sample.created_at AS created_at
+    %(values)s
 FROM survey_sample
 INNER JOIN (
     SELECT
         account_id,
+        campaign_id,
         MAX(created_at) AS last_updated_at
     FROM survey_sample
-    WHERE survey_sample.campaign_id = %(campaign_id)d AND
-          survey_sample.is_frozen
-          %(before_clause)s
-          %(filter_out_testing)s
-    GROUP BY account_id) AS last_updates
+    WHERE survey_sample.is_frozen
+          %(campaign_clause)s
+          %(date_range_clause)s
+          %(extra_clause)s
+    GROUP BY account_id, campaign_id) AS last_updates
 ON survey_sample.account_id = last_updates.account_id AND
+   survey_sample.campaign_id = last_updates.campaign_id AND
    survey_sample.created_at = last_updates.last_updated_at
 WHERE survey_sample.is_frozen
-""" % {'campaign_id': campaign.pk,
-       'before_clause': before_clause,
-       'filter_out_testing': filter_out_testing}
+      %(campaign_clause)s
+ORDER BY survey_sample.created_at DESC
+""" % {'values': values,
+       'campaign_clause': campaign_clause,
+       'date_range_clause': date_range_clause,
+       'extra_clause': extra_clause}
+        print("XXX sql_query=%s" % str(sql_query))
         return self.raw(sql_query)
 
     def get_score(self, sample):
@@ -423,20 +460,19 @@ class Sample(models.Model):
         help_text="Date/time of creation (in ISO format)")
     updated_at = models.DateTimeField(auto_now_add=True,
         help_text="Date/time of last update (in ISO format)")
+    time_spent = models.DurationField(default=datetime.timedelta,
+        help_text="Total recorded time to complete the survey")
     campaign = models.ForeignKey(Campaign, null=True, on_delete=models.PROTECT)
     account = models.ForeignKey(settings.ACCOUNT_MODEL,
         null=True, on_delete=models.PROTECT, related_name='samples')
     is_frozen = models.BooleanField(default=False,
         help_text="When True, answers to that sample cannot be updated.")
-    extra = get_extra_field_class()(null=True)
+    extra = get_extra_field_class()(null=True, blank=True,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     def __str__(self):
         return str(self.slug)
 
-    @property
-    def time_spent(self):
-        return self.updated_at - self.created_at
-
     @property
     def nb_answers(self):
         #pylint:disable=attribute-defined-outside-init
@@ -481,7 +517,7 @@ class Sample(models.Model):
             sample=self,
             question__campaigns__in=[self.campaign.pk]).order_by(
             'question__enumeratedquestions__rank').annotate(
-                rank=models.F('question__enumeratedquestions__rank'))
+                _rank=models.F('question__enumeratedquestions__rank'))
         return queryset
 
 
@@ -534,10 +570,31 @@ class Answer(models.Model):
         return str(self.question.content)
 
     @property
-    def as_text_value(self):
-        if self.unit.system in Unit.NUMERICAL_SYSTEMS:
-            return self.measured
-        return Choice.objects.get(pk=self.measured).text
+    def measured_text(self):
+        #pylint:disable=attribute-defined-outside-init
+        if not hasattr(self, '_measured_text'):
+            if self.unit.system in Unit.NUMERICAL_SYSTEMS:
+                self._measured_text = str(self.measured)
+            else:
+                self._measured_text = Choice.objects.get(
+                    pk=self.measured, unit=self.unit).text
+        return self._measured_text
+
+    @property
+    def is_equiv_default_unit(self):
+        """
+        Returns True if the answer unit is equivalent
+        to the question default unit.
+        """
+        #pylint:disable=attribute-defined-outside-init
+        if not hasattr(self, '_is_equiv_default_unit'):
+            if self.unit == self.question.default_unit:
+                self._is_equiv_default_unit = True
+            else:
+                self._is_equiv_default_unit = \
+                    UnitEquivalences.objects.are_equiv(
+                        self.unit, self.question.default_unit)
+        return self._is_equiv_default_unit
 
     @property
     def rank(self):
@@ -601,7 +658,7 @@ class EditablePredicate(models.Model):
     selector = models.CharField(max_length=255)
 
     def __str__(self):
-        return '%s-%d' % (self.portfolio.slug, int(self.rank))
+        return '%s-%d' % (self.editable_filter.slug, int(self.rank))
 
     def as_kwargs(self):
         kwargs = {}
@@ -652,7 +709,8 @@ class Matrix(SlugTitleMixin, models.Model):
     cohorts = models.ManyToManyField(EditableFilter, related_name='matrices')
     cut = models.ForeignKey(EditableFilter,
         null=True, on_delete=models.SET_NULL, related_name='cuts')
-    extra = get_extra_field_class()(null=True)
+    extra = get_extra_field_class()(null=True, blank=True,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     def __str__(self):
         return str(self.slug)
@@ -673,6 +731,8 @@ class Portfolio(models.Model):
     campaign = models.ForeignKey('Campaign', null=True,
         on_delete=models.PROTECT, db_index=True,
         related_name='portfolios')
+    extra = get_extra_field_class()(null=True, blank=True,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     class Meta:
         unique_together = (('grantee', 'account', 'campaign'),)
@@ -686,13 +746,17 @@ class PortfolioDoubleOptInQuerySet(models.QuerySet):
     def by_invoice_keys(self, invoice_keys):
         return self.filter(invoice_key__in=invoice_keys)
 
-    def by_grantee(self, account, start_at=None, ends_at=None, until=None):
+    def by_grantee(self, account, campaign=None,
+                   start_at=None, ends_at=None, until=None):
         """
         Returns the portfolio grant/request for `account` that were
         created in the period [start_at, ends_at[ (when specified)
         and that extend beyond `until` (when specified).
         """
+        #pylint:disable=too-many-arguments
         kwargs = {}
+        if campaign:
+            kwargs.update({'campaign': campaign})
         if start_at:
             kwargs.update({'created_at__gte': start_at})
         if ends_at:
@@ -701,40 +765,46 @@ class PortfolioDoubleOptInQuerySet(models.QuerySet):
             kwargs.update({'ends_at__gte': until})
         return self.filter(grantee=account, **kwargs)
 
-    def requested(self, account, start_at=None, ends_at=None, until=None):
+    def requested(self, account, campaign=None,
+                  start_at=None, ends_at=None, until=None):
         """
         Returns the portfolio requests made by `account` that were
         created in the period [start_at, ends_at[ (when specified)
         and that extend beyond `ends_at` (when specified).
         """
-        return self.by_grantee(account,
+        #pylint:disable=too-many-arguments
+        return self.by_grantee(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until).filter(state__in=(
             PortfolioDoubleOptIn.OPTIN_REQUEST_INITIATED,
             PortfolioDoubleOptIn.OPTIN_REQUEST_ACCEPTED,
             PortfolioDoubleOptIn.OPTIN_REQUEST_DENIED))
 
 
-    def unsolicited(self, account, start_at=None, ends_at=None, until=None):
+    def unsolicited(self, account, campaign=None,
+                    start_at=None, ends_at=None, until=None):
         """
         Returns the portfolio grants where `account` is the benefiary that were
         created in the period [start_at, ends_at[ (when specified)
         and that extend beyond `ends_at` (when specified).
         """
-        return self.by_grantee(account,
+        #pylint:disable=too-many-arguments
+        return self.by_grantee(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until).filter(state__in=(
             PortfolioDoubleOptIn.OPTIN_GRANT_INITIATED,
             PortfolioDoubleOptIn.OPTIN_GRANT_ACCEPTED,
             PortfolioDoubleOptIn.OPTIN_GRANT_DENIED))
 
 
-    def accepted(self, account, start_at=None, ends_at=None, until=None):
+    def accepted(self, account, campaign=None,
+                 start_at=None, ends_at=None, until=None):
         """
         Returns the portfolio grant/request where `account` is the benefiary
         that were accepted and which had been created in the period
         [start_at, ends_at[ (when specified) and that extend beyond
         `ends_at` (when specified).
         """
-        return self.by_grantee(account,
+        #pylint:disable=too-many-arguments
+        return self.by_grantee(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until).filter(state__in=(
             PortfolioDoubleOptIn.OPTIN_GRANT_ACCEPTED,
             PortfolioDoubleOptIn.OPTIN_REQUEST_ACCEPTED))
@@ -748,20 +818,28 @@ class PortfolioDoubleOptInManager(models.Manager):
     def by_invoice_keys(self, invoice_keys):
         return self.get_queryset().by_invoice_keys(invoice_keys)
 
-    def by_grantee(self, account, start_at=None, ends_at=None, until=None):
-        return self.get_queryset().by_grantee(account,
+    def by_grantee(self, account, campaign=None,
+                   start_at=None, ends_at=None, until=None):
+        #pylint:disable=too-many-arguments
+        return self.get_queryset().by_grantee(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until)
 
-    def requested(self, account, start_at=None, ends_at=None, until=None):
-        return self.get_queryset().requested(account,
+    def requested(self, account, campaign=None,
+                  start_at=None, ends_at=None, until=None):
+        #pylint:disable=too-many-arguments
+        return self.get_queryset().requested(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until)
 
-    def unsolicited(self, account, start_at=None, ends_at=None, until=None):
-        return self.get_queryset().unsolicited(account,
+    def unsolicited(self, account, campaign=None,
+                    start_at=None, ends_at=None, until=None):
+        #pylint:disable=too-many-arguments
+        return self.get_queryset().unsolicited(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until)
 
-    def accepted(self, account, start_at=None, ends_at=None, until=None):
-        return self.get_queryset().accepted(account,
+    def accepted(self, account,  campaign=None,
+                 start_at=None, ends_at=None, until=None):
+        #pylint:disable=too-many-arguments
+        return self.get_queryset().accepted(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until)
 
 
@@ -830,6 +908,8 @@ class PortfolioDoubleOptIn(models.Model):
     initiated_by = models.ForeignKey(settings.AUTH_USER_MODEL,
         on_delete=models.CASCADE)
     verification_key = models.CharField(max_length=40, null=True, unique=True)
+    extra = get_extra_field_class()(null=True, blank=True,
+        help_text=_("Extra meta data (can be stringify JSON)"))
 
     # To connect with payment provider.
     invoice_key = models.CharField(max_length=40, null=True)
@@ -858,10 +938,12 @@ class PortfolioDoubleOptIn(models.Model):
         with transaction.atomic():
             self.create_portfolios()
             self.state = PortfolioDoubleOptIn.OPTIN_GRANT_ACCEPTED
+            self.verification_key = None
             self.save()
 
     def request_accepted(self):
         with transaction.atomic():
             self.create_portfolios()
             self.state = PortfolioDoubleOptIn.OPTIN_REQUEST_ACCEPTED
+            self.verification_key = None
             self.save()
