@@ -17,9 +17,6 @@
 }(typeof self !== 'undefined' ? self : this, function (exports, jQuery) {
 
 
-/** Formats a date shown to the user.
-*/
-const DATE_FORMAT = 'MMM DD, YYYY';
 const DESC_SORT_PRE = '-';
 
 
@@ -32,7 +29,7 @@ var messagesMixin = {
     data: function() {
         return {
             messagesElement: '#messages-content',
-            scrollToTopOnMessages: true
+            scrollToTopOnMessages: true,
         }
     },
     methods: {
@@ -158,6 +155,120 @@ var messagesMixin = {
     }
 };
 
+
+/** compute outdated based on params.
+ */
+var paramsMixin = {
+    data: function(){
+        var data = {
+            autoreload: true,
+            lastGetParams: {},
+            params: {
+                // The following dates will be stored as `String` objects
+                // as oppossed to `moment` or `Date` objects because this
+                // is how form fields input="date" will update them.
+                start_at: null,
+                ends_at: null,
+                // The timezone for both start_at and ends_at.
+                timezone: 'local'
+            },
+        }
+        if( this.$dateRange ) {
+            if( this.$dateRange.start_at ) {
+                data.params['start_at'] = this.$dateRange.start_at;
+            }
+            if( this.$dateRange.ends_at ) {
+                data.params['ends_at'] = this.$dateRange.ends_at;
+            }
+            if( this.$dateRange.timezone ) {
+                data.params['timezone'] = this.$dateRange.timezone;
+            }
+        }
+        return data;
+    },
+    methods: {
+        asDateInputField: function(dateISOString) {
+            const dateValue = moment(dateISOString);
+            return dateValue.isValid() ? dateValue.format("YYYY-MM-DD") : null;
+        },
+        asDateISOString: function(dateInputField) {
+            const dateValue = moment(dateInputField, "YYYY-MM-DD");
+            return dateValue.isValid() ? dateValue.toISOString() : null;
+        },
+        getParams: function(excludes){
+            var vm = this;
+            var params = {};
+            for( var key in vm.params ) {
+                if( vm.params.hasOwnProperty(key) && vm.params[key] ) {
+                    if( excludes && key in excludes ) continue;
+                    params[key] = vm.params[key];
+                }
+            }
+            return params;
+        },
+    },
+    computed: {
+        _start_at: {
+            get: function() {
+                return this.asDateInputField(this.params.start_at);
+            },
+            set: function(newVal) {
+                if( newVal ) {
+                    // The setter might be call with `newVal === null`
+                    // when the date is incorrect (ex: 09/31/2022).
+                    this.$set(this.params, 'start_at',
+                        this.asDateISOString(newVal));
+                    if( this.autoreload && this.outdated ) this.get();
+                }
+            }
+        },
+        _ends_at: {
+            get: function() {
+                // form field input="date" will expect ends_at as a String
+                // but will literally cut the hour part regardless of timezone.
+                // We don't want an empty list as a result.
+                // If we use moment `endOfDay` we get 23:59:59 so we
+                // add a full day instead. It seemed clever to run the following
+                // code but that prevented entering the year part in the input
+                // field (as oppossed to use the widget).
+                //
+                // const dateValue = moment(this.params.ends_at).add(1,'days');
+                // return dateValue.isValid() ? dateValue.format("YYYY-MM-DD") : null;
+                return this.asDateInputField(this.params.ends_at);
+            },
+            set: function(newVal) {
+                if( newVal ) {
+                    // The setter might be call with `newVal === null`
+                    // when the date is incorrect (ex: 09/31/2022).
+                    this.$set(this.params, 'ends_at',
+                        this.asDateISOString(newVal));
+                    if( this.autoreload && this.outdated ) this.get();
+                }
+            }
+        },
+        outdated: function() {
+            var vm = this;
+            const params = vm.getParams();
+            for( var key in vm.lastGetParams ) {
+                if( vm.lastGetParams.hasOwnProperty(key) ) {
+                    if( vm.lastGetParams[key] !== params[key] ) {
+                        return true;
+                    }
+                }
+            }
+            for( var key in params ) {
+                if( params.hasOwnProperty(key) ) {
+                    if( params[key] !== vm.lastGetParams[key] ) {
+                        return true;
+                    }
+                }
+            }
+            return false;
+        }
+    }
+};
+
+
 /** A wrapper around jQuery ajax functions that adds authentication
     parameters as necessary.
 
@@ -165,8 +276,15 @@ var messagesMixin = {
 */
 var httpRequestMixin = {
     mixins: [
-        messagesMixin
+        messagesMixin,
+        paramsMixin
     ],
+// XXX conflitcs when params defined as props
+//    data: function() {
+//        return {
+//            params: {}
+//        }
+//    },
     // basically a wrapper around jQuery ajax functions
     methods: {
 
@@ -212,16 +330,50 @@ var httpRequestMixin = {
         _safeUrl: function(base, path) {
             if( !path ) return base;
 
-            if( base && base[base.length - 1] == '/') {
-                if( path && path[0] == '/') {
-                    return base + path.substring(1);
+            const parts = [base].concat(
+                ( typeof path === 'string' ) ? [path] : path);
+            var cleanParts = [];
+            var start, end;
+            for( var idx = 0; idx < parts.length; ++idx ) {
+                const part = parts[idx];
+                for( start = 0; start < part.length; ++start ) {
+                    if( part[start] !== '/') {
+                        break;
+                    }
+                }
+                for( end = part.length - 1; end >= 0; --end ) {
+                    if( part[end] !== '/') {
+                        break;
+                    }
+                }
+                if( start < end ) {
+                    cleanParts.push(part.slice(start, end + 1));
+                } else {
+                    cleanParts.push(part);
                 }
-                return base + path;
             }
-            if( path && path[0] == '/') {
-                return base + path;
+            var cleanUrl = cleanParts[0];
+            for( idx = 1; idx < cleanParts.length; ++idx ) {
+                cleanUrl += '/' + cleanParts[idx];
             }
-            return base + '/' + path;
+            return cleanUrl.startsWith('http') ? cleanUrl[0] : '/' + cleanUrl;
+        },
+
+        getQueryString: function(excludes){
+            var vm = this;
+            var sep = "";
+            var result = "";
+            var params = vm.getParams(excludes);
+            for( var key in params ) {
+                if( params.hasOwnProperty(key) ) {
+                    result += sep + key + '=' + params[key].toString();
+                    sep = "&";
+                }
+            }
+            if( result ) {
+                result = '?' + result;
+            }
+            return result;
         },
 
         /** This method generates a GET HTTP request to `url` with a query
@@ -680,30 +832,29 @@ var httpRequestMixin = {
                 failureCallback = vm.showErrorMessages;
             }
             for(var idx = 0; idx < queryArray.length; ++idx ) {
-                ajaxCalls.push(function () {
-                    return $.ajax({
-                        method: queryArray[idx].method,
-                        url: queryArray[idx].url,
-                        data: JSON.stringify(queryArray[idx].data),
-                        beforeSend: function(xhr, settings) {
-                            var authToken = vm._getAuthToken();
-                            if( authToken ) {
-                                xhr.setRequestHeader("Authorization",
-                                                     "Bearer " + authToken);
-                            } else {
-                                if( !vm._csrfSafeMethod(settings.type) ) {
-                                    var csrfToken = vm._getCSRFToken();
-                                    if( csrfToken ) {
-                                        xhr.setRequestHeader("X-CSRFToken", csrfToken);
-                                    }
+                ajaxCalls.push($.ajax({
+                    method: queryArray[idx].method,
+                    url: queryArray[idx].url,
+                    data: JSON.stringify(queryArray[idx].data),
+                    beforeSend: function(xhr, settings) {
+                        var authToken = vm._getAuthToken();
+                        if( authToken ) {
+                            xhr.setRequestHeader("Authorization",
+                                                 "Bearer " + authToken);
+                        } else {
+                            if( !vm._csrfSafeMethod(settings.type) ) {
+                                var csrfToken = vm._getCSRFToken();
+                                if( csrfToken ) {
+                                    xhr.setRequestHeader("X-CSRFToken", csrfToken);
                                 }
                             }
-                        },
-                        contentType: 'application/json',
-                    });
-                }());
+                        }
+                    },
+                    contentType: 'application/json',
+                }));
             }
-            jQuery.when(ajaxCalls).done(successCallback).fail(failureCallback);
+            jQuery.when.apply(jQuery, ajaxCalls).done(successCallback).fail(
+                failureCallback);
         },
     }
 }
@@ -1020,38 +1171,10 @@ var itemListMixin = {
                     count: 0
                 },
                 mergeResults: false,
-                params: {
-                    // The following dates will be stored as `String` objects
-                    // as oppossed to `moment` or `Date` objects because this
-                    // is how uiv-date-picker will update them.
-                    start_at: null,
-                    ends_at: null,
-                    // The timezone for both start_at and ends_at.
-                    timezone: 'local'
-                },
                 getCb: null,
                 getCompleteCb: null,
                 getBeforeCb: null,
             }
-            if( this.$dateRange ) {
-                if( this.$dateRange.start_at ) {
-                    data.params['start_at'] = moment(
-                        this.$dateRange.start_at).format("YYYY-MM-DD");
-                }
-                if( this.$dateRange.ends_at ) {
-                    // uiv-date-picker will expect ends_at as a String
-                    // but DATE_FORMAT will literally cut the hour part,
-                    // regardless of timezone. We don't want an empty list
-                    // as a result.
-                    // If we use moment `endOfDay` we get 23:59:59 so we
-                    // add a full day instead.
-                    data.params['ends_at'] = moment(
-                        this.$dateRange.ends_at).add(1,'days').format("YYYY-MM-DD");
-                }
-                if( this.$dateRange.timezone ) {
-                    data.params['timezone'] = this.$dateRange.timezone;
-                }
-            }
             return data;
         },
         get: function(){
@@ -1094,38 +1217,8 @@ var itemListMixin = {
             if(vm[vm.getBeforeCb]){
                 vm[vm.getBeforeCb]();
             }
-            vm.reqGet(vm.url, vm.getParams(), cb);
-        },
-        getParams: function(excludes){
-            var vm = this;
-            var params = {};
-            for( var key in vm.params ) {
-                if( vm.params.hasOwnProperty(key) && vm.params[key] ) {
-                    if( excludes && key in excludes ) continue;
-                    if( key === 'start_at' || key === 'ends_at' ) {
-                        params[key] = moment(vm.params[key], "YYYY-MM-DD").toISOString();
-                    } else {
-                        params[key] = vm.params[key];
-                    }
-                }
-            }
-            return params;
-        },
-        getQueryString: function(excludes){
-            var vm = this;
-            var sep = "";
-            var result = "";
-            var params = vm.getParams(excludes);
-            for( var key in params ) {
-                if( params.hasOwnProperty(key) ) {
-                    result += sep + key + '=' + params[key].toString();
-                    sep = "&";
-                }
-            }
-            if( result ) {
-                result = '?' + result;
-            }
-            return result;
+            vm.lastGetParams = vm.getParams();
+            vm.reqGet(vm.url, vm.lastGetParams, cb);
         },
     },
 };
@@ -1149,7 +1242,7 @@ var TypeAhead = Vue.extend({
     },
     methods: {
         activeClass: function activeClass(index) {
-            return {active: this.current === index};
+            return this.current === index ? " active" : "";
         },
 
         cancel: function() {},
@@ -1253,9 +1346,10 @@ var TypeAhead = Vue.extend({
 
     // attach properties to the exports object to define
     // the exported module properties.
-    exports.messagesMixin = messagesMixin;
     exports.httpRequestMixin = httpRequestMixin;
-    exports.itemMixin = itemMixin;
     exports.itemListMixin = itemListMixin;
+    exports.itemMixin = itemMixin;
+    exports.messagesMixin = messagesMixin;
+    exports.paramsMixin = paramsMixin;
     exports.TypeAhead = TypeAhead;
 }));
